@@ -1,5 +1,5 @@
 /*
- * $Id: DefaultMuleContext.java 22179 2011-06-15 12:15:00Z dirk.olmes $
+ * $Id: DefaultMuleContext.java 22968 2011-09-16 17:50:42Z dfeist $
  * --------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
  *
@@ -23,6 +23,7 @@ import org.mule.api.context.WorkManager;
 import org.mule.api.context.notification.ServerNotification;
 import org.mule.api.context.notification.ServerNotificationListener;
 import org.mule.api.endpoint.EndpointFactory;
+import org.mule.api.exception.RollbackSourceCallback;
 import org.mule.api.exception.SystemExceptionHandler;
 import org.mule.api.expression.ExpressionManager;
 import org.mule.api.lifecycle.Disposable;
@@ -36,7 +37,6 @@ import org.mule.api.registry.RegistrationException;
 import org.mule.api.registry.Registry;
 import org.mule.api.security.SecurityManager;
 import org.mule.api.store.ListableObjectStore;
-import org.mule.api.transaction.RollbackMethod;
 import org.mule.api.transaction.TransactionManagerFactory;
 import org.mule.client.DefaultLocalMuleClient;
 import org.mule.config.DefaultMuleConfiguration;
@@ -50,18 +50,25 @@ import org.mule.lifecycle.MuleContextLifecycleManager;
 import org.mule.management.stats.AllStatistics;
 import org.mule.registry.DefaultRegistryBroker;
 import org.mule.registry.MuleRegistryHelper;
+import org.mule.transport.DefaultPollingController;
+import org.mule.transport.PollingController;
 import org.mule.util.ApplicationShutdownSplashScreen;
 import org.mule.util.ApplicationStartupSplashScreen;
 import org.mule.util.ServerShutdownSplashScreen;
 import org.mule.util.ServerStartupSplashScreen;
 import org.mule.util.SplashScreen;
+import org.mule.util.UUID;
 import org.mule.util.queue.QueueManager;
 
 import java.io.Serializable;
 import java.util.Collection;
+import java.util.HashMap;
+import java.util.Map;
+import java.util.Set;
 
 import javax.resource.spi.work.WorkListener;
 import javax.transaction.TransactionManager;
+import javax.xml.namespace.QName;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
@@ -116,6 +123,14 @@ public class DefaultMuleContext implements MuleContext
     /** Global exception handler which handles "system" exceptions (i.e., when no message is involved). */
     protected SystemExceptionHandler exceptionListener;
 
+    private String clusterId = "";
+
+    private int clusterNodeId;
+
+    private PollingController pollingController = new DefaultPollingController();
+
+    private Map<QName, Set<Object>> configurationAnnotations;
+
     public DefaultMuleContext(MuleConfiguration config,
                               WorkManager workManager,
                               WorkListener workListener,
@@ -135,7 +150,8 @@ public class DefaultMuleContext implements MuleContext
         registryBroker = createRegistryBroker();
         muleRegistryHelper = createRegistryHelper(registryBroker);
         localMuleClient = new DefaultLocalMuleClient(this);
-        exceptionListener = new DefaultSystemExceptionStrategy(this, true);
+        exceptionListener = new DefaultSystemExceptionStrategy(this);
+        configurationAnnotations = new HashMap<QName, Set<Object>>();
     }
 
     protected DefaultRegistryBroker createRegistryBroker()
@@ -643,7 +659,7 @@ public class DefaultMuleContext implements MuleContext
         return localMuleClient;
     }
 
-    public void handleException(Exception e, RollbackMethod rollbackMethod)
+    public void handleException(Exception e, RollbackSourceCallback rollbackMethod)
     {
         if (e instanceof MessagingException)
         {
@@ -681,4 +697,47 @@ public class DefaultMuleContext implements MuleContext
         checkLifecycleForPropertySet(name, Initialisable.PHASE_NAME);
         registryBroker.registerObject(name, store);
     }
+
+    public String getClusterId()
+    {
+        return clusterId;
+    }
+
+    public void setClusterId(String clusterId)
+    {
+        this.clusterId = clusterId;
+    }
+
+    public int getClusterNodeId()
+    {
+        return clusterNodeId;
+    }
+
+    public void setClusterNodeId(int clusterNodeId)
+    {
+        this.clusterNodeId = clusterNodeId;
+    }
+
+    public void setPollingController(PollingController pollingController)
+    {
+        this.pollingController = pollingController;
+    }
+
+    @Override
+    public boolean isPrimaryPollingInstance()
+    {
+        return pollingController.isPrimaryPollingInstance();
+    }
+
+    @Override
+    public String getUniqueIdString()
+    {
+        return clusterNodeId + "-" + UUID.getUUID();
+    }
+
+    @Override
+    public Map<QName, Set<Object>> getConfigurationAnnotations()
+    {
+        return configurationAnnotations;
+    }
 }
