@@ -1,5 +1,5 @@
 /*
- * $Id: ImmutableEndpoint.java 22039 2011-05-30 19:12:17Z dfeist $
+ * $Id: ImmutableEndpoint.java 22969 2011-09-16 18:02:51Z dfeist $
  * --------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
  *
@@ -20,6 +20,7 @@ import org.mule.api.security.EndpointSecurityFilter;
 import org.mule.api.transaction.TransactionConfig;
 import org.mule.api.transformer.Transformer;
 import org.mule.api.transport.Connector;
+import org.mule.processor.AbstractRedeliveryPolicy;
 
 import java.io.Serializable;
 import java.util.List;
@@ -217,6 +218,11 @@ public interface ImmutableEndpoint extends Serializable, NamedObject
      * Return the mime type defined for the endpoint, if any
      */
     String getMimeType();
-    
+
+    /**
+     * Return the endpoint's redelivery policy, if any
+     */
+    AbstractRedeliveryPolicy getRedeliveryPolicy();
+
     boolean isDisableTransportTransformer();
 }
