@@ -1,7 +1,7 @@
 /*
- * $Id: ComponentDefinitionParser.java 14826 2009-05-28 09:25:21Z rossmason $
+ * $Id: ComponentDefinitionParser.java 23230 2011-10-21 12:13:44Z svacas $
  * --------------------------------------------------------------------------------------
- * Copyright (c) MuleSource, Inc.  All rights reserved.  http://www.mulesource.com
+ * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
  *
  * The software in this package is published under the terms of the CPAL v1.0
  * license, a copy of which has been included with this distribution in the
@@ -13,8 +13,6 @@ package org.mule.config.spring.parsers.specific;
 import org.mule.config.spring.parsers.AbstractMuleBeanDefinitionParser;
 import org.mule.config.spring.parsers.generic.ChildDefinitionParser;
 
-import org.springframework.beans.factory.support.BeanDefinitionBuilder;
-import org.springframework.beans.factory.xml.ParserContext;
 import org.w3c.dom.Element;
 
 public class ComponentDefinitionParser extends ChildDefinitionParser
@@ -22,17 +20,22 @@ public class ComponentDefinitionParser extends ChildDefinitionParser
 
     public ComponentDefinitionParser(Class clazz)
     {
-        super("component", clazz);
-        this.singleton = true;
+        super("messageProcessor", clazz);
+        this.singleton = false;
         addIgnored(AbstractMuleBeanDefinitionParser.ATTRIBUTE_CLASS);
     }
 
     @Override
-    protected void parseChild(Element element, ParserContext parserContext, BeanDefinitionBuilder builder)
+    public String getPropertyName(Element e)
     {
-        Element parent = (Element) element.getParentNode();
-        String serviceName = parent.getAttribute(ATTRIBUTE_NAME);
-        builder.addPropertyReference("service", serviceName);
-        super.parseChild(element, parserContext, builder);
+        String parent = e.getParentNode().getLocalName().toLowerCase();
+        if ("service".equals(parent) || "custom-service".equals(parent))
+        {
+            return "component";
+        }
+        else
+        {
+            return super.getPropertyName(e);
+        }
     }
 }
