@@ -1,5 +1,5 @@
 /*
- * $Id: DefaultInboundEndpoint.java 21522 2011-03-10 01:03:57Z dzapata $
+ * $Id: DefaultInboundEndpoint.java 22969 2011-09-16 18:02:51Z dfeist $
  * --------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
  *
@@ -30,6 +30,7 @@ import org.mule.api.transaction.TransactionConfig;
 import org.mule.api.transport.Connector;
 import org.mule.config.MuleManifest;
 import org.mule.config.i18n.CoreMessages;
+import org.mule.processor.AbstractRedeliveryPolicy;
 import org.mule.transport.ConnectException;
 import org.mule.transport.polling.MessageProcessorPollingMessageReceiver;
 
@@ -57,6 +58,7 @@ public class DefaultInboundEndpoint extends AbstractEndpoint implements InboundE
                                   String endpointBuilderName,
                                   MuleContext muleContext,
                                   RetryPolicyTemplate retryPolicyTemplate,
+                                  AbstractRedeliveryPolicy redeliveryPolicy,
                                   EndpointMessageProcessorChainFactory messageProcessorsFactory,
                                   List <MessageProcessor> messageProcessors,
                                   List <MessageProcessor> responseMessageProcessors,
@@ -66,7 +68,7 @@ public class DefaultInboundEndpoint extends AbstractEndpoint implements InboundE
         super(connector, endpointUri, name, properties, 
             transactionConfig, deleteUnacceptedMessage,
             messageExchangePattern, responseTimeout, initialState, endpointEncoding, 
-            endpointBuilderName, muleContext, retryPolicyTemplate,  messageProcessorsFactory, 
+            endpointBuilderName, muleContext, retryPolicyTemplate, redeliveryPolicy, messageProcessorsFactory,
             messageProcessors, responseMessageProcessors, disableTransportTransformer,
             mimeType);
     }
@@ -209,4 +211,5 @@ public class DefaultInboundEndpoint extends AbstractEndpoint implements InboundE
         // services get mixed up after deserialization of events
         return System.identityHashCode(this);
     }
+    
 }
