@@ -1,7 +1,7 @@
 /*
- * $Id: BankService.java 10669 2008-02-01 15:27:01Z romikk $
+ * $Id: BankService.java 19496 2010-09-09 16:16:02Z aperepel $
  * --------------------------------------------------------------------------------------
- * Copyright (c) MuleSource, Inc.  All rights reserved.  http://www.mulesource.com
+ * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
  *
  * The software in this package is published under the terms of the CPAL v1.0
  * license, a copy of which has been included with this distribution in the
@@ -10,13 +10,16 @@
 
 package org.mule.example.loanbroker.bank;
 
-import org.mule.example.loanbroker.messages.LoanBrokerQuoteRequest;
-import org.mule.example.loanbroker.messages.LoanQuote;
+import org.mule.example.loanbroker.message.LoanBrokerQuoteRequest;
+import org.mule.example.loanbroker.model.LoanQuote;
+
+import javax.jws.WebService;
 
 /**
  * <code>BankService</code> is a representation of a bank form which to obtain loan
  * quotes.
  */
+@WebService
 public interface BankService
 {
     LoanQuote getLoanQuote(LoanBrokerQuoteRequest request);
