@@ -1,7 +1,7 @@
 /*
- * $Id: RemoteDispatcher.java 13675 2008-12-30 02:19:43Z dfeist $
+ * $Id: RemoteDispatcher.java 23497 2011-12-18 11:46:14Z dfeist $
  * --------------------------------------------------------------------------------------
- * Copyright (c) MuleSource, Inc.  All rights reserved.  http://www.mulesource.com
+ * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
  *
  * The software in this package is published under the terms of the CPAL v1.0
  * license, a copy of which has been included with this distribution in the
@@ -10,27 +10,15 @@
 
 package org.mule.module.client;
 
-import java.io.ByteArrayInputStream;
-import java.io.ByteArrayOutputStream;
-import java.io.InputStream;
-import java.util.Map;
-
-import org.apache.commons.lang.SerializationUtils;
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
 import org.mule.DefaultMuleEvent;
 import org.mule.DefaultMuleMessage;
-import org.mule.DefaultMuleSession;
-import org.mule.MuleServer;
-import org.mule.MuleSessionHandler;
-import org.mule.NullSessionHandler;
-import org.mule.RegistryContext;
+import org.mule.MessageExchangePattern;
 import org.mule.RequestContext;
 import org.mule.api.FutureMessageResult;
+import org.mule.api.MuleContext;
 import org.mule.api.MuleEvent;
 import org.mule.api.MuleException;
 import org.mule.api.MuleMessage;
-import org.mule.api.MuleSession;
 import org.mule.api.config.MuleProperties;
 import org.mule.api.endpoint.EndpointBuilder;
 import org.mule.api.endpoint.EndpointFactory;
@@ -40,6 +28,7 @@ import org.mule.api.lifecycle.Disposable;
 import org.mule.api.security.Credentials;
 import org.mule.api.transformer.wire.WireFormat;
 import org.mule.api.transport.DispatchException;
+import org.mule.client.DefaultLocalMuleClient.MuleClientFlowConstruct;
 import org.mule.module.client.i18n.ClientMessages;
 import org.mule.module.client.remoting.RemoteDispatcherException;
 import org.mule.module.client.remoting.ServerHandshake;
@@ -47,12 +36,20 @@ import org.mule.module.client.remoting.UnsupportedWireFormatException;
 import org.mule.module.client.remoting.notification.RemoteDispatcherNotification;
 import org.mule.security.MuleCredentials;
 import org.mule.transformer.TransformerUtils;
-import org.mule.transport.AbstractConnector;
+import org.mule.transport.NullPayload;
 import org.mule.util.ClassUtils;
 import org.mule.util.IOUtils;
+import org.mule.util.SerializationUtils;
 
-import edu.emory.mathcs.backport.java.util.concurrent.Callable;
-import edu.emory.mathcs.backport.java.util.concurrent.Executor;
+import java.io.ByteArrayInputStream;
+import java.io.ByteArrayOutputStream;
+import java.io.InputStream;
+import java.util.Map;
+import java.util.concurrent.Callable;
+import java.util.concurrent.Executor;
+
+import org.apache.commons.logging.Log;
+import org.apache.commons.logging.LogFactory;
 
 /**
  * <code>RemoteDispatcher</code> is used to make and receive requests to a remote
@@ -74,6 +71,7 @@ public class RemoteDispatcher implements Disposable
     private OutboundEndpoint asyncServerEndpoint;
     private OutboundEndpoint syncServerEndpoint;
     private Credentials credentials = null;
+    private MuleContext muleContext;
 
     /**
      * an ExecutorService for async messages (optional)
@@ -85,20 +83,21 @@ public class RemoteDispatcher implements Disposable
      */
     private WireFormat wireFormat;
 
-    protected RemoteDispatcher(String endpoint, Credentials credentials) throws MuleException
+    protected RemoteDispatcher(String endpoint, Credentials credentials, MuleContext muleContext) throws MuleException
     {
-        this(endpoint);
+        this(endpoint, muleContext);
         this.credentials = credentials;
     }
 
-    protected RemoteDispatcher(String endpoint) throws MuleException
+    protected RemoteDispatcher(String endpoint, MuleContext muleContext) throws MuleException
     {
-        EndpointFactory endpointFactory = RegistryContext.getRegistry().lookupEndpointFactory();
+        this.muleContext = muleContext;
+        EndpointFactory endpointFactory = muleContext.getEndpointFactory();
         asyncServerEndpoint = endpointFactory.getOutboundEndpoint(endpoint);
 
         EndpointBuilder endpointBuilder = endpointFactory.getEndpointBuilder(endpoint);
-        endpointBuilder.setSynchronous(true);
-        syncServerEndpoint = RegistryContext.getRegistry().lookupEndpointFactory().getOutboundEndpoint(
+        endpointBuilder.setExchangePattern(MessageExchangePattern.REQUEST_RESPONSE);
+        syncServerEndpoint = muleContext.getEndpointFactory().getOutboundEndpoint(
             endpointBuilder);
 
         wireFormat = requestWireFormat();
@@ -106,9 +105,15 @@ public class RemoteDispatcher implements Disposable
 
     protected WireFormat requestWireFormat() throws MuleException
     {
-        MuleMessage msg = new DefaultMuleMessage(ServerHandshake.SERVER_HANDSHAKE_PROPERTY);
-        MuleMessage result = syncServerEndpoint.send(new DefaultMuleEvent(msg, syncServerEndpoint,
-                new DefaultMuleSession(msg, new NullSessionHandler(), MuleServer.getMuleContext()), true));
+        MuleMessage msg = new DefaultMuleMessage(ServerHandshake.SERVER_HANDSHAKE_PROPERTY, muleContext);
+        MuleMessage result = null;
+
+        MuleEvent resultEvent = syncServerEndpoint.process(new DefaultMuleEvent(msg,
+            MessageExchangePattern.REQUEST_RESPONSE, new MuleClientFlowConstruct(muleContext)));
+        if (resultEvent != null)
+        {
+            result = resultEvent.getMessage();
+        }
 
         if(result==null)
         {
@@ -119,7 +124,7 @@ public class RemoteDispatcher implements Disposable
         try
         {
             ByteArrayInputStream in = new ByteArrayInputStream(result.getPayloadAsBytes());
-            handshake = (ServerHandshake) SerializationUtils.deserialize(in);
+            handshake = (ServerHandshake) SerializationUtils.deserialize(in, muleContext);
         }
         catch (Exception e)
         {
@@ -131,6 +136,7 @@ public class RemoteDispatcher implements Disposable
             WireFormat wf = (WireFormat)ClassUtils.instanciateClass(handshake.getWireFormatClass(),
                     ClassUtils.NO_ARGS, getClass());
 
+            wf.setMuleContext(muleContext);
             return wf;
         }
         catch (Exception e)
@@ -211,7 +217,7 @@ public class RemoteDispatcher implements Disposable
             }
         };
 
-        FutureMessageResult result = new FutureMessageResult(callable);
+        FutureMessageResult result = new FutureMessageResult(callable, muleContext);
 
         if (asyncExecutor != null)
         {
@@ -220,7 +226,7 @@ public class RemoteDispatcher implements Disposable
 
         if (transformers != null)
         {
-            result.setTransformers(TransformerUtils.getTransformers(transformers));
+            result.setTransformers(TransformerUtils.getTransformers(transformers, muleContext));
         }
 
         result.execute();
@@ -235,8 +241,8 @@ public class RemoteDispatcher implements Disposable
 
     public MuleMessage sendRemote(String endpoint, Object payload, Map messageProperties) throws MuleException
     {
-        return doToRemote(endpoint, payload, messageProperties, true, 
-            MuleServer.getMuleContext().getConfiguration().getDefaultResponseTimeout());
+        return doToRemote(endpoint, payload, messageProperties, true,
+            muleContext.getConfiguration().getDefaultResponseTimeout());
     }
 
     public void dispatchRemote(String endpoint, Object payload, Map messageProperties) throws MuleException
@@ -256,7 +262,7 @@ public class RemoteDispatcher implements Disposable
             }
         };
 
-        FutureMessageResult result = new FutureMessageResult(callable);
+        FutureMessageResult result = new FutureMessageResult(callable, muleContext);
 
         if (asyncExecutor != null)
         {
@@ -271,7 +277,10 @@ public class RemoteDispatcher implements Disposable
     {
         RemoteDispatcherNotification action = new RemoteDispatcherNotification(null, RemoteDispatcherNotification.ACTION_RECEIVE, endpoint);
         action.setProperty(MuleProperties.MULE_REMOTE_SYNC_PROPERTY, "true");
-        action.setProperty(MuleProperties.MULE_EVENT_TIMEOUT_PROPERTY, new Long(timeout));
+        if (timeout != MuleEvent.TIMEOUT_NOT_SET_VALUE)
+        {
+            action.setProperty(MuleProperties.MULE_EVENT_TIMEOUT_PROPERTY, new Long(timeout));
+        }
         return dispatchAction(action, true, timeout);
     }
 
@@ -286,7 +295,7 @@ public class RemoteDispatcher implements Disposable
             }
         };
 
-        FutureMessageResult result = new FutureMessageResult(callable);
+        FutureMessageResult result = new FutureMessageResult(callable, muleContext);
 
         if (asyncExecutor != null)
         {
@@ -302,13 +311,13 @@ public class RemoteDispatcher implements Disposable
                                              Map messageProperties,
                                              boolean synchronous) throws MuleException
     {
-        MuleMessage message = new DefaultMuleMessage(payload, messageProperties);
-        message.setBooleanProperty(MuleProperties.MULE_REMOTE_SYNC_PROPERTY, synchronous);
+        MuleMessage message = new DefaultMuleMessage(payload, messageProperties, muleContext);
+        message.setOutboundProperty(MuleProperties.MULE_REMOTE_SYNC_PROPERTY, synchronous);
         setCredentials(message);
         RemoteDispatcherNotification action = new RemoteDispatcherNotification(message, RemoteDispatcherNotification.ACTION_INVOKE,
             "mule://" + component);
-        return dispatchAction(action, synchronous, 
-            MuleServer.getMuleContext().getConfiguration().getDefaultResponseTimeout());
+        return dispatchAction(action, synchronous,
+            muleContext.getConfiguration().getDefaultResponseTimeout());
     }
 
     protected MuleMessage doToRemote(String endpoint,
@@ -317,8 +326,8 @@ public class RemoteDispatcher implements Disposable
                                     boolean synchronous,
                                     int timeout) throws MuleException
     {
-        MuleMessage message = new DefaultMuleMessage(payload, messageProperties);
-        message.setProperty(MuleProperties.MULE_REMOTE_SYNC_PROPERTY, String.valueOf(synchronous));
+        MuleMessage message = new DefaultMuleMessage(payload, messageProperties, muleContext);
+        message.setOutboundProperty(MuleProperties.MULE_REMOTE_SYNC_PROPERTY, String.valueOf(synchronous));
         setCredentials(message);
         RemoteDispatcherNotification action = new RemoteDispatcherNotification(message, (synchronous
                         ? RemoteDispatcherNotification.ACTION_SEND : RemoteDispatcherNotification.ACTION_DISPATCH), endpoint);
@@ -338,8 +347,8 @@ public class RemoteDispatcher implements Disposable
         {
             serverEndpoint = asyncServerEndpoint;
         }
-        MuleMessage serializeMessage = new DefaultMuleMessage(action);
-        
+        MuleMessage serializeMessage = new DefaultMuleMessage(action, muleContext);
+
         updateContext(serializeMessage, serverEndpoint, synchronous);
 
         ByteArrayOutputStream out = new ByteArrayOutputStream();
@@ -350,18 +359,16 @@ public class RemoteDispatcher implements Disposable
 
         if (message == null)
         {
-            message = new DefaultMuleMessage(payload);
+            message = new DefaultMuleMessage(payload, muleContext);
         }
         else
         {
-            message = new DefaultMuleMessage(payload, message);
+            message = new DefaultMuleMessage(payload, message, muleContext);
         }
 
         message.addProperties(action.getProperties());
-        MuleSession session = new DefaultMuleSession(message,
-            ((AbstractConnector)serverEndpoint.getConnector()).getSessionHandler(), MuleServer.getMuleContext());
 
-        MuleEvent event = new DefaultMuleEvent(message, serverEndpoint, session, true);
+        MuleEvent event = new DefaultMuleEvent(message, serverEndpoint.getExchangePattern(), null);
         event.setTimeout(timeout);
         if (logger.isDebugEnabled())
         {
@@ -369,49 +376,48 @@ public class RemoteDispatcher implements Disposable
                          + serverEndpoint.toString() + " . Event is: " + event);
         }
 
-        MuleMessage result;
+
+        MuleMessage result = null;
 
         try
         {
-            if (synchronous)
+            MuleEvent resultEvent = serverEndpoint.process(event);
+            if (resultEvent != null)
             {
-                result = serverEndpoint.send(event);
-            }
-            else
-            {
-                serverEndpoint.dispatch(event);
-                return null;
+                result = resultEvent.getMessage();
             }
 
-            if (result != null)
+            if (result != null && result.getPayload() != null)
             {
-                if (result.getPayload() != null)
+                if (result.getPayload() instanceof NullPayload)
                 {
-                    Object response;
-                    if (result.getPayload() instanceof InputStream)
-                    {
-                        byte[] b = IOUtils.toByteArray((InputStream)result.getPayload());
-                        if(b.length==0) return null;
-                        ByteArrayInputStream in = new ByteArrayInputStream(b);
-                        response = wireFormat.read(in);
-                    }
-                    else
-                    {
-                        ByteArrayInputStream in = new ByteArrayInputStream(result.getPayloadAsBytes());
-                        response = wireFormat.read(in);
-                    }
-
-                    if (response instanceof RemoteDispatcherNotification)
-                    {
-                        response = ((RemoteDispatcherNotification)response).getMessage();
-                    }
-                    return (MuleMessage)response;
+                    return null;
                 }
+
+                Object response;
+                if (result.getPayload() instanceof InputStream)
+                {
+                    byte[] b = IOUtils.toByteArray((InputStream)result.getPayload());
+                    if(b.length==0) return null;
+                    ByteArrayInputStream in = new ByteArrayInputStream(b);
+                    response = wireFormat.read(in);
+                }
+                else
+                {
+                    ByteArrayInputStream in = new ByteArrayInputStream(result.getPayloadAsBytes());
+                    response = wireFormat.read(in);
+                }
+
+                if (response instanceof RemoteDispatcherNotification)
+                {
+                    response = ((RemoteDispatcherNotification)response).getMessage();
+                }
+                return (MuleMessage)response;
             }
         }
         catch (Exception e)
         {
-            throw new DispatchException(event.getMessage(), event.getEndpoint(), e);
+            throw new DispatchException(event, serverEndpoint, e);
         }
 
         if (logger.isDebugEnabled())
@@ -432,8 +438,8 @@ public class RemoteDispatcher implements Disposable
     {
         if (credentials != null)
         {
-            message.setProperty(MuleProperties.MULE_USER_PROPERTY, MuleCredentials.createHeader(
-                credentials.getUsername(), credentials.getPassword()));
+            message.setOutboundProperty(MuleProperties.MULE_USER_PROPERTY, MuleCredentials.createHeader(
+                    credentials.getUsername(), credentials.getPassword()));
         }
     }
 
@@ -450,8 +456,6 @@ public class RemoteDispatcher implements Disposable
     protected void updateContext(MuleMessage message, ImmutableEndpoint endpoint, boolean synchronous)
         throws MuleException
     {
-
-        RequestContext.setEvent(new DefaultMuleEvent(message, endpoint, new DefaultMuleSession(message,
-            new MuleSessionHandler(), MuleServer.getMuleContext()), synchronous));
+        RequestContext.setEvent(new DefaultMuleEvent(message, endpoint.getExchangePattern(), null));
     }
 }
