@@ -1,5 +1,5 @@
 /*
- * $Id: DefaultMuleApplication.java 22252 2011-06-23 06:15:55Z dirk.olmes $
+ * $Id: DefaultMuleApplication.java 23467 2011-12-08 14:56:06Z pablo.kraan $
  * --------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
  *
@@ -28,6 +28,7 @@ import org.mule.module.launcher.ApplicationMuleContextBuilder;
 import org.mule.module.launcher.ConfigChangeMonitorThreadFactory;
 import org.mule.module.launcher.DefaultMuleSharedDomainClassLoader;
 import org.mule.module.launcher.DeploymentInitException;
+import org.mule.module.launcher.DeploymentListener;
 import org.mule.module.launcher.DeploymentService;
 import org.mule.module.launcher.DeploymentStartException;
 import org.mule.module.launcher.DeploymentStopException;
@@ -76,9 +77,22 @@ public class DefaultMuleApplication implements Application
 
     protected String[] absoluteResourcePaths;
 
+    protected DeploymentListener deploymentListener;
+
     protected DefaultMuleApplication(ApplicationDescriptor appDesc)
     {
         this.descriptor = appDesc;
+        this.deploymentListener = new NullDeploymentListener();
+    }
+
+    public void setDeploymentListener(DeploymentListener deploymentListener)
+    {
+        if (deploymentListener == null)
+        {
+            throw new IllegalArgumentException("Deployment listener cannot be null");
+        }
+
+        this.deploymentListener = deploymentListener;
     }
 
     @Override
@@ -325,15 +339,41 @@ public class DefaultMuleApplication implements Application
         {
             logger.info(miniSplash(String.format("Redeploying app '%s'", descriptor.getAppName())));
         }
-        dispose();
+
+        String appName = getAppName();
+
+        deploymentListener.onDeploymentStart(appName);
+        try
+        {
+            dispose();
+
+            deploymentListener.onUndeploymentSuccess(appName);
+        }
+        catch (RuntimeException e)
+        {
+            deploymentListener.onUndeploymentFailure(appName, e);
+
+            throw e;
+        }
+
         install();
 
         // update thread with the fresh new classloader just created during the install phase
         final ClassLoader cl = getDeploymentClassLoader();
         Thread.currentThread().setContextClassLoader(cl);
 
-        init();
-        start();
+        deploymentListener.onDeploymentStart(appName);
+        try
+        {
+            init();
+            start();
+
+            deploymentListener.onDeploymentSuccess(appName);
+        }
+        catch(Throwable cause)
+        {
+            deploymentListener.onDeploymentFailure(appName, cause);
+        }
 
         // release the ref
         Thread.currentThread().setContextClassLoader(null);
