@@ -1,5 +1,5 @@
 /*
- * $Id: EventCorrelator.java 21939 2011-05-18 13:32:09Z aperepel $
+ * $Id: EventCorrelator.java 23513 2011-12-20 16:01:56Z pablo.lagreca $
  * --------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
  *
@@ -7,6 +7,7 @@
  * license, a copy of which has been included with this distribution in the
  * LICENSE.txt file.
  */
+
 package org.mule.routing.correlation;
 
 import org.mule.api.MessagingException;
@@ -14,6 +15,7 @@ import org.mule.api.MuleContext;
 import org.mule.api.MuleEvent;
 import org.mule.api.MuleException;
 import org.mule.api.MuleMessageCollection;
+import org.mule.api.config.MuleProperties;
 import org.mule.api.construct.FlowConstruct;
 import org.mule.api.lifecycle.Startable;
 import org.mule.api.lifecycle.Stoppable;
@@ -21,23 +23,30 @@ import org.mule.api.processor.MessageProcessor;
 import org.mule.api.routing.MessageInfoMapping;
 import org.mule.api.routing.RoutingException;
 import org.mule.api.service.Service;
+import org.mule.api.store.ListableObjectStore;
+import org.mule.api.store.ObjectAlreadyExistsException;
+import org.mule.api.store.ObjectDoesNotExistException;
+import org.mule.api.store.ObjectStore;
+import org.mule.api.store.ObjectStoreException;
+import org.mule.api.store.ObjectStoreManager;
+import org.mule.api.transaction.TransactionCallback;
 import org.mule.config.i18n.CoreMessages;
 import org.mule.context.notification.RoutingNotification;
 import org.mule.routing.EventGroup;
+import org.mule.routing.EventProcessingThread;
+import org.mule.transaction.TransactionTemplate;
+import org.mule.transaction.TransactionTemplateFactory;
 import org.mule.util.StringMessageUtils;
 import org.mule.util.concurrent.ThreadNameHelper;
 import org.mule.util.monitor.Expirable;
 import org.mule.util.monitor.ExpiryMonitor;
 
+import java.io.Serializable;
 import java.text.MessageFormat;
 import java.util.ArrayList;
 import java.util.List;
-import java.util.Map;
-import java.util.concurrent.ConcurrentHashMap;
-import java.util.concurrent.ConcurrentMap;
 import java.util.concurrent.TimeUnit;
 
-import org.apache.commons.collections.buffer.BoundedFifoBuffer;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 
@@ -65,12 +74,12 @@ public class EventCorrelator implements Startable, Stoppable
      * agregated, keyed by message id. There will be one response message for every
      * EventGroup.
      */
-    protected final ConcurrentMap eventGroups = new ConcurrentHashMap();
+    protected ListableObjectStore<EventGroup> eventGroups;
 
     protected final Object groupsLock = new Object();
 
     // @GuardedBy groupsLock
-    protected final BoundedFifoBuffer processedGroups = new BoundedFifoBuffer(MAX_PROCESSED_GROUPS);
+    protected ObjectStore<Long> processedGroups = null;
 
     private long timeout = -1; // undefined
 
@@ -87,16 +96,26 @@ public class EventCorrelator implements Startable, Stoppable
     /**
      * A map with keys = group id and values = group creation time
      */
-    private Map expiredAndDispatchedGroups = new ConcurrentHashMap();
+    private ListableObjectStore<Long> expiredAndDispatchedGroups = null;
 
     private EventCorrelator.ExpiringGroupMonitoringThread expiringGroupMonitoringThread;
     private final String name;
 
-    public EventCorrelator(EventCorrelatorCallback callback, MessageProcessor timeoutMessageProcessor, MessageInfoMapping messageInfoMapping, MuleContext muleContext, String flowConstructName)
+    private final boolean persistentStores;
+    private final String storePrefix;
+
+    public EventCorrelator(EventCorrelatorCallback callback,
+                           MessageProcessor timeoutMessageProcessor,
+                           MessageInfoMapping messageInfoMapping,
+                           MuleContext muleContext,
+                           String flowConstructName,
+                           boolean persistentStores,
+                           String storePrefix)
     {
         if (callback == null)
         {
-            throw new IllegalArgumentException(CoreMessages.objectIsNull("EventCorrelatorCallback").getMessage());
+            throw new IllegalArgumentException(CoreMessages.objectIsNull("EventCorrelatorCallback")
+                .getMessage());
         }
         if (messageInfoMapping == null)
         {
@@ -110,18 +129,39 @@ public class EventCorrelator implements Startable, Stoppable
         this.messageInfoMapping = messageInfoMapping;
         this.muleContext = muleContext;
         this.timeoutMessageProcessor = timeoutMessageProcessor;
-        name = String.format("%s%s.event.correlator", ThreadNameHelper.getPrefix(muleContext), flowConstructName);
+        this.persistentStores = persistentStores;
+        this.storePrefix = storePrefix;
+        name = String.format("%s%s.event.correlator", ThreadNameHelper.getPrefix(muleContext),
+            flowConstructName);
+        ObjectStoreManager objectStoreManager = muleContext.getRegistry().get(
+            MuleProperties.OBJECT_STORE_MANAGER);
+        expiredAndDispatchedGroups = (ListableObjectStore<Long>) objectStoreManager.getObjectStore(
+            storePrefix + ".expiredAndDispatchedGroups", persistentStores);
+        processedGroups = (ListableObjectStore<Long>) objectStoreManager.getObjectStore(storePrefix
+                                                                                        + ".processedGroups",
+            persistentStores, MAX_PROCESSED_GROUPS, -1, 1000);
+        eventGroups = (ListableObjectStore<EventGroup>) objectStoreManager.getObjectStore(storePrefix
+                                                                                          + ".eventGroups",
+            persistentStores);
     }
 
     public void forceGroupExpiry(String groupId) throws MessagingException
     {
-        if (eventGroups.get(groupId) != null)
+        try
         {
-            handleGroupExpiry((EventGroup) eventGroups.get(groupId));
+            if (eventGroups.retrieve(groupId) != null)
+            {
+                handleGroupExpiry((EventGroup) eventGroups.retrieve(groupId));
+            }
+            else
+            {
+                addProcessedGroup(groupId);
+            }
         }
-        else
+        catch (ObjectStoreException e)
         {
-            addProcessedGroup(groupId);
+            // TODO improve this
+            throw new MessagingException(null, e);
         }
     }
 
@@ -135,9 +175,9 @@ public class EventCorrelator implements Startable, Stoppable
             try
             {
                 logger.trace(String.format("Received async reply message for correlationID: %s%n%s%n%s",
-                                           groupId,
-                                           StringMessageUtils.truncate(StringMessageUtils.toString(event.getMessage().getPayload()), 200, false),
-                                           StringMessageUtils.headersToString(event.getMessage())));
+                    groupId, StringMessageUtils.truncate(
+                        StringMessageUtils.toString(event.getMessage().getPayload()), 200, false),
+                    StringMessageUtils.headersToString(event.getMessage())));
             }
             catch (Exception e)
             {
@@ -168,49 +208,70 @@ public class EventCorrelator implements Startable, Stoppable
                 }
             }
 
-            if (isGroupAlreadyProcessed(groupId))
+            try
             {
-                if (logger.isDebugEnabled())
+                if (isGroupAlreadyProcessed(groupId))
                 {
-                    logger.debug("An event was received for an event group that has already been processed, " +
-                                 "this is probably because the async-reply timed out. Correlation Id is: " + groupId +
-                                 ". Dropping event");
+                    if (logger.isDebugEnabled())
+                    {
+                        logger.debug("An event was received for an event group that has already been processed, "
+                                     + "this is probably because the async-reply timed out. Correlation Id is: "
+                                     + groupId + ". Dropping event");
+                    }
+                    // Fire a notification to say we received this message
+                    muleContext.fireNotification(new RoutingNotification(event.getMessage(),
+                        event.getMessageSourceURI().toString(),
+                        RoutingNotification.MISSED_AGGREGATION_GROUP_EVENT));
+                    return null;
                 }
-                //Fire a notification to say we received this message
-                muleContext.fireNotification(new RoutingNotification(event.getMessage(),
-                                                                     event.getEndpoint().getEndpointURI().toString(),
-                                                                     RoutingNotification.MISSED_AGGREGATION_GROUP_EVENT));
-                return null;
+            }
+            catch (ObjectStoreException e)
+            {
+                throw new RoutingException(event, timeoutMessageProcessor, e);
             }
 
             // check for an existing group first
-            EventGroup group = this.getEventGroup(groupId);
+            EventGroup group;
+            try
+            {
+                group = this.getEventGroup(groupId);
+            }
+            catch (ObjectStoreException e)
+            {
+                throw new RoutingException(event, timeoutMessageProcessor, e);
+            }
 
             // does the group exist?
             if (group == null)
             {
                 // ..apparently not, so create a new one & add it
-                group = this.addEventGroup(callback.createEventGroup(event, groupId));
+                try
+                {
+                    group = this.addEventGroup(callback.createEventGroup(event, groupId));
+                }
+                catch (ObjectStoreException e)
+                {
+                    throw new RoutingException(event, timeoutMessageProcessor, e);
+                }
             }
 
             // ensure that only one thread at a time evaluates this EventGroup
             synchronized (groupsLock)
             {
-                // make sure no other thread removed the group in the meantime
-                if (group != this.getEventGroup(groupId))
-                {
-                    // if that is the (rare) case, spin
-                    lookupMiss = true;
-                    continue;
-                }
-
                 if (logger.isDebugEnabled())
                 {
                     logger.debug("Adding event to aggregator group: " + groupId);
                 }
 
                 // add the incoming event to the group
-                group.addEvent(event);
+                try
+                {
+                    group.addEvent(event);
+                }
+                catch (ObjectStoreException e)
+                {
+                    throw new RoutingException(event, timeoutMessageProcessor, e);
+                }
 
                 // check to see if the event group is ready to be aggregated
                 if (callback.shouldAggregateEvents(group))
@@ -218,10 +279,23 @@ public class EventCorrelator implements Startable, Stoppable
                     // create the response event
                     MuleEvent returnEvent = callback.aggregateEvents(group);
                     returnEvent.getMessage().setCorrelationId(groupId);
+                    String rootId = group.getCommonRootId();
+                    if (rootId != null)
+                    {
+                        returnEvent.getMessage().setMessageRootId(rootId);
+                    }
 
                     // remove the eventGroup as no further message will be received
                     // for this group once we aggregate
-                    this.removeEventGroup(group);
+                    try
+                    {
+                        this.removeEventGroup(group);
+                        group.clear();
+                    }
+                    catch (ObjectStoreException e)
+                    {
+                        throw new RoutingException(event, timeoutMessageProcessor, e);
+                    }
 
                     return returnEvent;
                 }
@@ -233,43 +307,51 @@ public class EventCorrelator implements Startable, Stoppable
         }
     }
 
-    protected EventGroup getEventGroup(String groupId)
+    protected EventGroup getEventGroup(String groupId) throws ObjectStoreException
     {
-        return (EventGroup) eventGroups.get(groupId);
-    }
-
-    protected EventGroup addEventGroup(EventGroup group)
-    {
-        EventGroup previous = (EventGroup) eventGroups.putIfAbsent(group.getGroupId(), group);
-        // a parallel thread might have removed the EventGroup already,
-        // therefore we need to validate our current reference
-        return (previous != null ? previous : group);
-    }
-
-    protected void removeEventGroup(EventGroup group)
-    {
-        final Object groupId = group.getGroupId();
-        eventGroups.remove(groupId);
-        addProcessedGroup(groupId);
-    }
-
-    protected void addProcessedGroup(Object id)
-    {
-        synchronized (groupsLock)
+        try
         {
-            if (processedGroups.isFull())
-            {
-                processedGroups.remove();
-            }
-            processedGroups.add(id);
+            return (EventGroup) eventGroups.retrieve(groupId);
+        }
+        catch (ObjectDoesNotExistException e)
+        {
+            return null;
         }
     }
 
-    protected boolean isGroupAlreadyProcessed(Object id)
+    protected EventGroup addEventGroup(EventGroup group) throws ObjectStoreException
+    {
+        try
+        {
+            eventGroups.store((Serializable) group.getGroupId(), group);
+            return group;
+        }
+        catch (ObjectAlreadyExistsException e)
+        {
+            return (EventGroup) eventGroups.retrieve((Serializable) group.getGroupId());
+        }
+    }
+
+    protected void removeEventGroup(EventGroup group) throws ObjectStoreException
+    {
+        final Object groupId = group.getGroupId();
+        eventGroups.remove((Serializable) groupId);
+        addProcessedGroup(groupId);
+    }
+
+    protected void addProcessedGroup(Object id) throws ObjectStoreException
     {
         synchronized (groupsLock)
         {
-            return processedGroups.contains(id);
+            processedGroups.store((Serializable) id, System.nanoTime());
+        }
+    }
+
+    protected boolean isGroupAlreadyProcessed(Object id) throws ObjectStoreException
+    {
+        synchronized (groupsLock)
+        {
+            return processedGroups.contains((Serializable) id);
         }
     }
 
@@ -295,26 +377,49 @@ public class EventCorrelator implements Startable, Stoppable
 
     protected void handleGroupExpiry(EventGroup group) throws MessagingException
     {
-        removeEventGroup(group);
-
-        final FlowConstruct service = group.toArray()[0].getFlowConstruct();
+        try
+        {
+            removeEventGroup(group);
+        }
+        catch (ObjectStoreException e)
+        {
+            throw new MessagingException(group.getMessageCollectionEvent(), e);
+        }
 
         if (isFailOnTimeout())
         {
-            final MuleMessageCollection messageCollection = group.toMessageCollection();
+            MuleMessageCollection messageCollection;
+            try
+            {
+                messageCollection = group.toMessageCollection();
+            }
+            catch (ObjectStoreException e)
+            {
+                throw new MessagingException(group.getMessageCollectionEvent(), e);
+            }
             muleContext.fireNotification(new RoutingNotification(messageCollection, null,
-                                                                 RoutingNotification.CORRELATION_TIMEOUT));
+                RoutingNotification.CORRELATION_TIMEOUT));
+            MuleEvent groupCollectionEvent = group.getMessageCollectionEvent();
+            try
+            {
+                group.clear();
+            }
+            catch (ObjectStoreException e)
+            {
+                logger.warn("Failed to clear group with id " + group.getGroupId()
+                            + " since underlying ObjectStore threw Exception:" + e.getMessage());
+            }
             throw new CorrelationTimeoutException(CoreMessages.correlationTimedOut(group.getGroupId()),
-                                                    group.getMessageCollectionEvent());
+                groupCollectionEvent);
         }
         else
         {
             if (logger.isDebugEnabled())
             {
                 logger.debug(MessageFormat.format(
-                        "Aggregator expired, but ''failOnTimeOut'' is false. Forwarding {0} events out of {1} " +
-                        "total for group ID: {2}", group.size(), group.expectedSize(), group.getGroupId()
-                ));
+                    "Aggregator expired, but ''failOnTimeOut'' is false. Forwarding {0} events out of {1} "
+                                    + "total for group ID: {2}", group.size(), group.expectedSize(),
+                    group.getGroupId()));
             }
 
             try
@@ -322,27 +427,30 @@ public class EventCorrelator implements Startable, Stoppable
                 if (!(group.getCreated() + groupTimeToLive < System.currentTimeMillis()))
                 {
                     MuleEvent newEvent = callback.aggregateEvents(group);
+                    group.clear();
                     newEvent.getMessage().setCorrelationId(group.getGroupId().toString());
 
-
-                    if (!expiredAndDispatchedGroups.containsKey(group.getGroupId()))
+                    if (!expiredAndDispatchedGroups.contains((Serializable) group.getGroupId()))
                     {
-                        // TODO which use cases would need a sync reply event returned?
+                        // TODO which use cases would need a sync reply event
+                        // returned?
                         if (timeoutMessageProcessor != null)
                         {
                             timeoutMessageProcessor.process(newEvent);
                         }
                         else
                         {
+                            final FlowConstruct service = group.toArray()[0].getFlowConstruct();
                             if (!(service instanceof Service))
                             {
-                                throw new UnsupportedOperationException("EventAggregator is only supported with Service");
+                                throw new UnsupportedOperationException(
+                                    "EventAggregator is only supported with Service");
                             }
 
                             ((Service) service).dispatchEvent(newEvent);
                         }
-                        expiredAndDispatchedGroups.put(group.getGroupId(),
-                                                       group.getCreated());
+                        expiredAndDispatchedGroups.store((Serializable) group.getGroupId(),
+                            group.getCreated());
                     }
                     else
                     {
@@ -380,105 +488,90 @@ public class EventCorrelator implements Startable, Stoppable
         }
     }
 
-
-    private final class ExpiringGroupMonitoringThread extends Thread implements Expirable
+    private final class ExpiringGroupMonitoringThread extends EventProcessingThread implements Expirable
     {
         private ExpiryMonitor expiryMonitor;
-
-        private volatile boolean stopRequested;
+        public static final long DELAY_TIME =  10;
 
         public ExpiringGroupMonitoringThread()
         {
-            setName(name);
-            this.expiryMonitor = new ExpiryMonitor(name, 1000 * 60);
-            //clean up every 30 minutes
+            super(name, DELAY_TIME);
+            this.expiryMonitor = new ExpiryMonitor(name, 1000 * 60, muleContext, true);
+            // clean up every 30 minutes
             this.expiryMonitor.addExpirable(1000 * 60 * 30, TimeUnit.MILLISECONDS, this);
         }
 
         /**
-         * Removes the elements in expiredAndDispatchedGroups when groupLife is reached
+         * Removes the elements in expiredAndDispatchedGroups when groupLife is
+         * reached
+         * 
+         * @throws ObjectStoreException
          */
         public void expired()
         {
-            for (Object o : expiredAndDispatchedGroups.keySet())
+            try
             {
-                Long time = (Long) expiredAndDispatchedGroups.get(o);
-                if (time + groupTimeToLive < System.currentTimeMillis())
+                for (Serializable o : expiredAndDispatchedGroups.allKeys())
                 {
-                    expiredAndDispatchedGroups.remove(o);
-                    logger.warn(MessageFormat.format("Discarding group {0}", o));
+                    Long time = (Long) expiredAndDispatchedGroups.retrieve(o);
+                    if (time + groupTimeToLive < System.currentTimeMillis())
+                    {
+                        expiredAndDispatchedGroups.remove(o);
+                        logger.warn(MessageFormat.format("Discarding group {0}", o));
+                    }
                 }
             }
+            catch (ObjectStoreException e)
+            {
+                logger.warn("Expiration of objects failed due to ObjectStoreException " + e + ".");
+            }
         }
 
-        public void release()
+        public void doRun()
         {
-            //no op
-        }
-
-        public void run()
-        {
-            while (true)
+            List<EventGroup> expired = new ArrayList<EventGroup>(1);
+            try
             {
-                if (stopRequested)
+                for (Serializable o : eventGroups.allKeys())
                 {
-                    logger.debug("Received request to stop expiring group monitoring");
-                    break;
-                }
-
-                List<EventGroup> expired = new ArrayList<EventGroup>(1);
-                for (Object o : eventGroups.values())
-                {
-                    EventGroup group = (EventGroup) o;
+                    EventGroup group = (EventGroup) eventGroups.retrieve(o);
                     if ((group.getCreated() + getTimeout() * MILLI_TO_NANO_MULTIPLIER) < System.nanoTime())
                     {
                         expired.add(group);
                     }
                 }
-                if (expired.size() > 0)
+            }
+            catch (ObjectStoreException e)
+            {
+                logger.warn("expiry failed dues to ObjectStoreException " + e);
+            }
+            if (expired.size() > 0)
+            {
+                for (Object anExpired : expired)
                 {
-                    for (Object anExpired : expired)
+                    final EventGroup group = (EventGroup) anExpired;
+                    TransactionTemplate<Void> exceptionHandlingTransactionTemplate = TransactionTemplateFactory.<Void>createExceptionHandlingTransactionTemplate(muleContext);
+                    try
                     {
-                        EventGroup group = (EventGroup) anExpired;
-                        try
+                        exceptionHandlingTransactionTemplate.execute(new TransactionCallback<Void>()
                         {
-                            handleGroupExpiry(group);
-                        }
-                        catch (MessagingException e)
-                        {
-                            e.getEvent().getFlowConstruct().getExceptionListener().handleException(e, e.getEvent());
-                        }
+                            @Override
+                            public Void doInTransaction() throws Exception
+                            {
+                                handleGroupExpiry(group);
+                                return null;
+                            }
+                        });
+                    }
+                    catch (MessagingException e)
+                    {
+                        //Already handled by TransactionTemplate
+                    }
+                    catch (Exception e)
+                    {
+                        muleContext.getExceptionListener().handleException(e);
                     }
                 }
-
-                try
-                {
-                    Thread.sleep(100);
-                }
-                catch (InterruptedException e)
-                {
-                    break;
-                }
-            }
-
-            logger.debug("Expiring group monitoring fully stopped");
-        }
-
-        /**
-         * Stops the monitoring of the expired groups.
-         */
-        public void stopProcessing()
-        {
-            logger.debug("Stopping expiring group monitoring");
-            stopRequested = true;
-
-            try
-            {
-                this.join();
-            }
-            catch (InterruptedException e)
-            {
-                // Ignoring
             }
         }
     }
