@@ -1,5 +1,5 @@
 /*
- * $Id: AbstractOutboundRouter.java 21939 2011-05-18 13:32:09Z aperepel $
+ * $Id: AbstractOutboundRouter.java 23513 2011-12-20 16:01:56Z pablo.lagreca $
  * --------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
  *
@@ -12,11 +12,7 @@ package org.mule.routing.outbound;
 
 import org.mule.DefaultMuleEvent;
 import org.mule.DefaultMuleMessage;
-import org.mule.api.MessagingException;
-import org.mule.api.MuleContext;
-import org.mule.api.MuleEvent;
-import org.mule.api.MuleException;
-import org.mule.api.MuleMessage;
+import org.mule.api.*;
 import org.mule.api.config.MuleProperties;
 import org.mule.api.construct.FlowConstruct;
 import org.mule.api.construct.FlowConstructAware;
@@ -40,6 +36,7 @@ import org.mule.processor.AbstractMessageProcessorOwner;
 import org.mule.routing.CorrelationMode;
 import org.mule.routing.DefaultRouterResultsHandler;
 import org.mule.transaction.TransactionTemplate;
+import org.mule.transaction.TransactionTemplateFactory;
 import org.mule.util.StringMessageUtils;
 import org.mule.util.SystemUtils;
 
@@ -92,27 +89,36 @@ public abstract class AbstractOutboundRouter extends AbstractMessageProcessorOwn
 
     public MuleEvent process(final MuleEvent event) throws MuleException
     {
-        TransactionTemplate<MuleEvent> tt = new TransactionTemplate<MuleEvent>(getTransactionConfig(),
-            muleContext);
-
+        TransactionTemplate<MuleEvent> tt = TransactionTemplateFactory.<MuleEvent>createNestedTransactionTemplate(getTransactionConfig(), muleContext);
         TransactionCallback<MuleEvent> cb = new TransactionCallback<MuleEvent>()
         {
             public MuleEvent doInTransaction() throws Exception
             {
-                return route(event);
+                try
+                {
+                    return route(event);
+                }
+                catch (RoutingException e)
+                {
+                    throw e;
+                }
+                catch (Exception e)
+                {
+                    throw new RoutingException(event, AbstractOutboundRouter.this, e);
+                }
             }
         };
         try
         {
             return tt.execute(cb);
         }
-        catch (RoutingException e)
+        catch (MuleException e)
         {
             throw e;
         }
         catch (Exception e)
         {
-            throw new RoutingException(event, this, e);
+            throw new DefaultMuleException(e);
         }
     }
 
@@ -129,7 +135,7 @@ public abstract class AbstractOutboundRouter extends AbstractMessageProcessorOwn
             awaitResponse = false;
         }
 
-        setMessageProperties(routedEvent.getSession().getFlowConstruct(), message, route);
+        setMessageProperties(routedEvent.getFlowConstruct(), message, route);
 
         if (logger.isDebugEnabled())
         {
@@ -466,8 +472,7 @@ public abstract class AbstractOutboundRouter extends AbstractMessageProcessorOwn
      */
     protected MuleEvent createEventToRoute(MuleEvent routedEvent, MuleMessage message, MessageProcessor route)
     {
-        MuleEvent event = new DefaultMuleEvent(message, routedEvent.getEndpoint(), routedEvent.getSession(), routedEvent.getProcessingTime());
-        return event;
+        return new DefaultMuleEvent(message, routedEvent, true);
     }
 
     /**
@@ -475,14 +480,13 @@ public abstract class AbstractOutboundRouter extends AbstractMessageProcessorOwn
      */
     protected MuleMessage cloneMessage(MuleMessage message)
     {
-        MuleMessage clonedMessage = new DefaultMuleMessage(message.getPayload(), message, muleContext);
-        return clonedMessage;
+        return new DefaultMuleMessage(message.getPayload(), message, muleContext);
     }
 
     /**
      * Propagates a number of internal system properties to handle correlation, session, etc. Note that in and
      * out params can be the same message object when not dealing with replies.
-     * 
+     *
      * @see #magicProperties
      */
     protected void propagateMagicProperties(MuleMessage in, MuleMessage out)
