@@ -1,5 +1,5 @@
 /*
- * $Id: MuleTestUtils.java 22156 2011-06-08 21:36:30Z dfeist $
+ * $Id: MuleTestUtils.java 23507 2011-12-20 12:57:52Z dfeist $
  * --------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
  *
@@ -282,8 +282,7 @@ public final class MuleTestUtils
             public ImmutableEndpoint getEndpoint(EndpointBuilder builder) throws MuleException
             {
                 builder.setExchangePattern(mep);
-                TransactionConfig txConfig = new MuleTransactionConfig();
-                txConfig.setAction(TransactionConfig.ACTION_BEGIN_OR_JOIN);
+                TransactionConfig txConfig = new MuleTransactionConfig(TransactionConfig.ACTION_BEGIN_OR_JOIN);
                 txConfig.setFactory(new TestTransactionFactory());
                 builder.setTransactionConfig(txConfig);
                 return context.getEndpointFactory().getInboundEndpoint(builder);
@@ -499,7 +498,7 @@ public final class MuleTestUtils
         final MuleMessageFactory factory = endpoint.getConnector().createMuleMessageFactory();
         final MuleMessage message = factory.create(data, endpoint.getEncoding());
 
-        return new DefaultMuleEvent(message, endpoint, session);
+        return new DefaultMuleEvent(message, endpoint, flowConstruct, session);
     }
 
     public static MuleEvent getTestEvent(Object data, MuleSession session, MuleContext context)
@@ -510,7 +509,7 @@ public final class MuleTestUtils
         final MuleMessageFactory factory = endpoint.getConnector().createMuleMessageFactory();
         final MuleMessage message = factory.create(data, endpoint.getEncoding());
 
-        return new DefaultMuleEvent(message, endpoint, session);
+        return new DefaultMuleEvent(message, endpoint, getTestService(context), session);
     }
 
     public static MuleEventContext getTestEventContext(Object data,
@@ -538,7 +537,7 @@ public final class MuleTestUtils
 
     public static MuleSession getTestSession(FlowConstruct flowConstruct, MuleContext context)
     {
-        return new DefaultMuleSession(flowConstruct, context);
+        return new DefaultMuleSession();
     }
 
     public static MuleSession getTestSession(MuleContext context)
@@ -627,7 +626,10 @@ public final class MuleTestUtils
 
     public static Mock getMockSession()
     {
-        return new Mock(MuleSession.class, "umoSession");
+        Mock session = new Mock(MuleSession.class, "umoSession");
+        session.matchAndReturn("getProperties", new HashMap<String, Object>());
+        return session;
+
     }
 
     public static Mock getMockMessageDispatcher()
