@@ -1,5 +1,5 @@
 /*
- * $Id: AbstractTransformer.java 21939 2011-05-18 13:32:09Z aperepel $
+ * $Id: AbstractTransformer.java 23214 2011-10-18 17:36:27Z dfeist $
  * --------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
  *
@@ -10,12 +10,12 @@
 
 package org.mule.transformer;
 
+import org.mule.api.AnnotatedObject;
 import org.mule.api.MuleContext;
 import org.mule.api.MuleEvent;
 import org.mule.api.MuleException;
 import org.mule.api.MuleMessage;
 import org.mule.api.config.MuleProperties;
-import org.mule.api.context.notification.MuleContextNotificationListener;
 import org.mule.api.endpoint.ImmutableEndpoint;
 import org.mule.api.lifecycle.InitialisationException;
 import org.mule.api.transformer.DataType;
@@ -24,8 +24,6 @@ import org.mule.api.transformer.TransformerException;
 import org.mule.api.transformer.TransformerMessagingException;
 import org.mule.config.i18n.CoreMessages;
 import org.mule.config.i18n.Message;
-import org.mule.context.notification.MuleContextNotification;
-import org.mule.context.notification.NotificationException;
 import org.mule.transformer.types.DataTypeFactory;
 import org.mule.transformer.types.SimpleDataType;
 import org.mule.transport.NullPayload;
@@ -37,10 +35,13 @@ import java.io.InputStream;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.List;
+import java.util.Map;
+import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.CopyOnWriteArrayList;
 
 import javax.activation.MimeType;
 import javax.activation.MimeTypeParseException;
+import javax.xml.namespace.QName;
 import javax.xml.transform.stream.StreamSource;
 
 import org.apache.commons.logging.Log;
@@ -51,7 +52,7 @@ import org.apache.commons.logging.LogFactory;
  * Transformations transform one object into another.
  */
 
-public abstract class AbstractTransformer implements Transformer, MuleContextNotificationListener<MuleContextNotification>
+public abstract class AbstractTransformer implements Transformer, AnnotatedObject
 {
     public static final DataType<MuleMessage> MULE_MESSAGE_DATA_TYPE = new SimpleDataType<MuleMessage>(MuleMessage.class);
 
@@ -99,6 +100,7 @@ public abstract class AbstractTransformer implements Transformer, MuleContextNot
      */
     protected String mimeType;
     protected String encoding;
+    private final Map<QName, Object> annotations = new ConcurrentHashMap<QName, Object>();
 
     /**
      * default constructor required for discovery
@@ -555,21 +557,21 @@ public abstract class AbstractTransformer implements Transformer, MuleContextNot
     public void setMuleContext(MuleContext context)
     {
         this.muleContext = context;
-        try
-        {
-            muleContext.registerListener(this);
-        }
-        catch (NotificationException e)
-        {
-            logger.error("failed to register context listener", e);
-        }
     }
 
-    public void onNotification(MuleContextNotification notification)
+    public final Object getAnnotation(QName name)
     {
-        if (notification.getAction() == MuleContextNotification.CONTEXT_DISPOSING)
-        {
-            this.dispose();
-        }
+        return annotations.get(name);
+    }
+
+    public final Map<QName, Object> getAnnotations()
+    {
+        return Collections.unmodifiableMap(annotations);
+    }
+
+    public synchronized final void setAnnotations(Map<QName, Object> newAnnotations)
+    {
+        annotations.clear();
+        annotations.putAll(newAnnotations);
     }
 }
