@@ -1,5 +1,5 @@
 /*
- * $Id: TransactedPollingMessageReceiver.java 21939 2011-05-18 13:32:09Z aperepel $
+ * $Id: TransactedPollingMessageReceiver.java 23513 2011-12-20 16:01:56Z pablo.lagreca $
  * --------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
  *
@@ -21,6 +21,7 @@ import org.mule.api.lifecycle.CreateException;
 import org.mule.api.transaction.TransactionCallback;
 import org.mule.api.transport.Connector;
 import org.mule.transaction.TransactionTemplate;
+import org.mule.transaction.TransactionTemplateFactory;
 
 import java.util.List;
 import java.util.concurrent.CountDownLatch;
@@ -107,15 +108,20 @@ public abstract class TransactedPollingMessageReceiver extends AbstractPollingMe
     @Override
     public void poll() throws Exception
     {
-        TransactionTemplate<Object> tt = new TransactionTemplate<Object>(endpoint.getTransactionConfig(), connector.getMuleContext());
+        TransactionTemplate<Object> tt = TransactionTemplateFactory.<Object>createMainTransactionTemplate(endpoint.getTransactionConfig(), connector.getMuleContext());
 
         if (this.isReceiveMessagesInTransaction())
         {
+            if (hasNoMessages())
+            {
+                return;
+            }
             // Receive messages and process them in a single transaction
             // Do not enable threading here, but several workers
             // may have been started
             TransactionCallback<Object> cb = new TransactionCallback<Object>()
             {
+                @Override
                 public Object doInTransaction() throws Exception
                 {
                     // this is not ideal, but jdbc receiver returns a list of maps, not List<MuleMessage>
@@ -157,6 +163,14 @@ public abstract class TransactedPollingMessageReceiver extends AbstractPollingMe
         }
     }
 
+    /**
+     * Return true if it can be determined that there are currently no messages to process
+     */
+    protected boolean hasNoMessages()
+    {
+        return  false;
+    }
+
     protected class MessageProcessorWorker implements Work, TransactionCallback
     {
         private final TransactionTemplate tt;
@@ -170,11 +184,13 @@ public abstract class TransactedPollingMessageReceiver extends AbstractPollingMe
             this.latch = latch;
         }
 
+        @Override
         public void release()
         {
             // nothing to do
         }
 
+        @Override
         public void run()
         {
             try
@@ -183,8 +199,7 @@ public abstract class TransactedPollingMessageReceiver extends AbstractPollingMe
             }
             catch (MessagingException e)
             {
-                MuleEvent event = ((MessagingException) e).getEvent();
-                event.getFlowConstruct().getExceptionListener().handleException(e, event);
+                //already managed by TransactionTemplate
             }
             catch (Exception e)
             {
@@ -196,6 +211,7 @@ public abstract class TransactedPollingMessageReceiver extends AbstractPollingMe
             }
         }
 
+        @Override
         public Object doInTransaction() throws Exception
         {
             processMessage(message);
