@@ -1,7 +1,7 @@
 /*
- * $Id: FileConnector.java 14826 2009-05-28 09:25:21Z rossmason $
+ * $Id: FileConnector.java 23502 2011-12-19 15:35:39Z dirk.olmes $
  * --------------------------------------------------------------------------------------
- * Copyright (c) MuleSource, Inc.  All rights reserved.  http://www.mulesource.com
+ * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
  *
  * The software in this package is published under the terms of the CPAL v1.0
  * license, a copy of which has been included with this distribution in the
@@ -10,22 +10,26 @@
 
 package org.mule.transport.file;
 
-import org.mule.api.MessagingException;
+import org.mule.api.DefaultMuleException;
+import org.mule.api.MuleContext;
+import org.mule.api.MuleEvent;
 import org.mule.api.MuleException;
 import org.mule.api.MuleMessage;
 import org.mule.api.config.MuleProperties;
-import org.mule.api.endpoint.ImmutableEndpoint;
+import org.mule.api.construct.FlowConstruct;
 import org.mule.api.endpoint.InboundEndpoint;
+import org.mule.api.endpoint.OutboundEndpoint;
+import org.mule.api.lifecycle.CreateException;
 import org.mule.api.lifecycle.InitialisationException;
-import org.mule.api.service.Service;
 import org.mule.api.transport.DispatchException;
-import org.mule.api.transport.MessageAdapter;
 import org.mule.api.transport.MessageReceiver;
+import org.mule.api.transport.MuleMessageFactory;
 import org.mule.config.i18n.CoreMessages;
 import org.mule.transformer.simple.ByteArrayToSerializable;
 import org.mule.transformer.simple.SerializableToByteArray;
 import org.mule.transport.AbstractConnector;
 import org.mule.transport.file.filters.FilenameWildcardFilter;
+import org.mule.transport.file.i18n.FileMessages;
 import org.mule.util.FileUtils;
 
 import java.io.File;
@@ -47,21 +51,19 @@ import org.apache.commons.logging.LogFactory;
 public class FileConnector extends AbstractConnector
 {
 
-    public static final String FILE = "file";
     private static Log logger = LogFactory.getLog(FileConnector.class);
 
+    public static final String FILE = "file";
+    private static final String DEFAULT_WORK_FILENAME_PATTERN = "#[function:uuid].#[function:systime].#[header:originalFilename]";
+
     // These are properties that can be overridden on the Receiver by the endpoint declaration
     // inbound only
-    public static final String PROPERTY_POLLING_FREQUENCY = "pollingFrequency";
     public static final String PROPERTY_FILE_AGE = "fileAge";
     public static final String PROPERTY_MOVE_TO_PATTERN = "moveToPattern";
     public static final String PROPERTY_MOVE_TO_DIRECTORY = "moveToDirectory";
     public static final String PROPERTY_READ_FROM_DIRECTORY = "readFromDirectoryName";
     // outbound only
     public static final String PROPERTY_OUTPUT_PATTERN = "outputPattern";
-    // apparently unused (once strange override code deleted)
-//    public static final String PROPERTY_DELETE_ON_READ = "autoDelete";
-//    public static final String PROPERTY_SERVICE_OVERRIDE = "serviceOverrides";
 
     // message properties
     public static final String PROPERTY_FILENAME = "filename";
@@ -69,10 +71,10 @@ public class FileConnector extends AbstractConnector
     public static final String PROPERTY_DIRECTORY = "directory";
     public static final String PROPERTY_WRITE_TO_DIRECTORY = "writeToDirectoryName";
     public static final String PROPERTY_FILE_SIZE = "fileSize";
+    public static final String PROPERTY_FILE_TIMESTAMP = "timestamp";
 
     public static final long DEFAULT_POLLING_FREQUENCY = 1000;
 
-
     /**
      * Time in milliseconds to poll. On each poll the poll() method is called
      */
@@ -84,6 +86,10 @@ public class FileConnector extends AbstractConnector
 
     private String moveToDirectoryName = null;
 
+    private String workDirectoryName = null;
+
+    private String workFileNamePattern = DEFAULT_WORK_FILENAME_PATTERN;
+
     private String readFromDirectoryName = null;
 
     private String outputPattern = null;
@@ -104,24 +110,33 @@ public class FileConnector extends AbstractConnector
 
     public FilenameParser filenameParser;
 
-    /*
-     * (non-Javadoc)
-     *
-     * @see org.mule.transport.AbstractConnector#doInitialise()
-     */
-    public FileConnector()
+    private boolean recursive = false;
+
+    public FileConnector(MuleContext context)
     {
-        super();
-        filenameParser = new SimpleFilenameParser();
+        super(context);
+        filenameParser = new ExpressionFilenameParser();
     }
 
+    @Override
+    protected void configureDispatcherPool()
+    {
+        if (isOutputAppend())
+        {
+            setMaxDispatchersActive(getDispatcherThreadingProfile().getMaxThreadsActive());
+        }
+        else
+        {
+            super.configureDispatcherPool();
+        }
+    }
 
     @Override
     public void setMaxDispatchersActive(int value)
     {
         if (isOutputAppend() && value != 1)
         {
-            logger.warn("MULE-1773: cannot configure maxDispatchersActive when using outputAppend.  New value not set");
+            logger.warn("MULE-1773: cannot configure maxDispatchersActive when using outputAppend. New value not set");
         }
         else
         {
@@ -130,7 +145,7 @@ public class FileConnector extends AbstractConnector
     }
 
     @Override
-    protected Object getReceiverKey(Service service, InboundEndpoint endpoint)
+    protected Object getReceiverKey(FlowConstruct flowConstruct, InboundEndpoint endpoint)
     {
         if (endpoint.getFilter() != null && endpoint.getFilter() instanceof FilenameWildcardFilter)
         {
@@ -150,7 +165,8 @@ public class FileConnector extends AbstractConnector
      * <li>pollingFrequency</li>
      * </ul>
      */
-    public MessageReceiver createReceiver(Service service, InboundEndpoint endpoint) throws Exception
+    @Override
+    public MessageReceiver createReceiver(FlowConstruct flowConstruct, InboundEndpoint endpoint) throws Exception
     {
         String readDir = endpoint.getEndpointURI().getAddress();
         if (null != getReadFromDirectory())
@@ -214,27 +230,12 @@ public class FileConnector extends AbstractConnector
                     logger.error("Failed to set fileAge", ex1);
                 }
             }
-
-            // this is surreal! what on earth was it useful for?
-            // Map srvOverride = (Map) props.get(PROPERTY_SERVICE_OVERRIDE);
-            // if (srvOverride != null)
-            // {
-            // if (serviceOverrides == null)
-            // {
-            // serviceOverrides = new Properties();
-            // }
-            // serviceOverrides.setProperty(MuleProperties.CONNECTOR_INBOUND_TRANSFORMER,
-            // NoActionTransformer.class.getName());
-            // serviceOverrides.setProperty(MuleProperties.CONNECTOR_OUTBOUND_TRANSFORMER,
-            // NoActionTransformer.class.getName());
-            // }
-
         }
 
         try
         {
-            return serviceDescriptor.createMessageReceiver(this, service, endpoint, new Object[]{readDir,
-                    moveTo, moveToPattern, new Long(polling)});
+            return serviceDescriptor.createMessageReceiver(this, flowConstruct, endpoint, new Object[]{readDir,
+                    moveTo, moveToPattern, Long.valueOf(polling)});
 
         }
         catch (Exception e)
@@ -258,8 +259,13 @@ public class FileConnector extends AbstractConnector
     public void setFilenameParser(FilenameParser filenameParser)
     {
         this.filenameParser = filenameParser;
+        if (filenameParser != null)
+        {
+            filenameParser.setMuleContext(muleContext);
+        }
     }
 
+    @Override
     protected void doDispose()
     {
         try
@@ -272,9 +278,14 @@ public class FileConnector extends AbstractConnector
         }
     }
 
-
+    @Override
     protected void doInitialise() throws InitialisationException
     {
+        if (filenameParser != null)
+        {
+            filenameParser.setMuleContext(muleContext);
+        }
+
         // MULE-1773: limit the number of dispatchers per endpoint to 1 until
         // there is a proper (Distributed)LockManager in place (MULE-2402).
         // We also override the setter to prevent "wrong" configuration for now.
@@ -284,21 +295,25 @@ public class FileConnector extends AbstractConnector
         }
     }
 
+    @Override
     protected void doConnect() throws Exception
     {
         // template method, nothing to do
     }
 
+    @Override
     protected void doDisconnect() throws Exception
     {
         // template method, nothing to do
     }
 
+    @Override
     protected void doStart() throws MuleException
     {
         // template method, nothing to do
     }
 
+    @Override
     protected void doStop() throws MuleException
     {
         if (outputStream != null)
@@ -314,89 +329,85 @@ public class FileConnector extends AbstractConnector
         }
     }
 
-    /**
-     * @return Returns the moveToDirectoryName.
-     */
     public String getMoveToDirectory()
     {
         return moveToDirectoryName;
     }
 
-    /**
-     * @param dir The moveToDirectoryName to set.
-     */
     public void setMoveToDirectory(String dir)
     {
         this.moveToDirectoryName = dir;
     }
 
-    /**
-     * @return Returns the outputAppend.
-     */
+    public void setWorkDirectory(String workDirectoryName) throws IOException
+    {
+        this.workDirectoryName = workDirectoryName;
+        if (workDirectoryName != null)
+        {
+            File workDirectory = FileUtils.openDirectory(workDirectoryName);
+            if (!workDirectory.canWrite())
+            {
+                throw new IOException(
+                        "Error on initialization, Work Directory '" + workDirectory +"' is not writeable");
+            }
+        }
+    }
+
+    public String getWorkDirectory()
+    {
+        return workDirectoryName;
+    }
+
+    public void setWorkFileNamePattern(String workFileNamePattern)
+    {
+        this.workFileNamePattern = workFileNamePattern;
+    }
+
+    public String getWorkFileNamePattern()
+    {
+        return workFileNamePattern;
+    }
+
     public boolean isOutputAppend()
     {
         return outputAppend;
     }
 
-    /**
-     * @param outputAppend The outputAppend to set.
-     */
     public void setOutputAppend(boolean outputAppend)
     {
         this.outputAppend = outputAppend;
     }
 
-    /**
-     * @return Returns the outputPattern.
-     */
     public String getOutputPattern()
     {
         return outputPattern;
     }
 
-    /**
-     * @param outputPattern The outputPattern to set.
-     */
     public void setOutputPattern(String outputPattern)
     {
         this.outputPattern = outputPattern;
     }
 
-    /**
-     * @return Returns the outputStream.
-     */
     public FileOutputStream getOutputStream()
     {
         return outputStream;
     }
 
-    /**
-     * @param outputStream The outputStream to set.
-     */
     public void setOutputStream(FileOutputStream outputStream)
     {
         this.outputStream = outputStream;
     }
 
-    /**
-     * @return Returns the pollingFrequency.
-     */
     public long getPollingFrequency()
     {
         return pollingFrequency;
     }
 
-    /**
-     * @param pollingFrequency The pollingFrequency to set.
-     */
     public void setPollingFrequency(long pollingFrequency)
     {
         this.pollingFrequency = pollingFrequency;
     }
 
-    /**
-     * @return Returns the fileAge.
-     */
     public long getFileAge()
     {
         return fileAge;
@@ -407,62 +418,44 @@ public class FileConnector extends AbstractConnector
         return checkFileAge;
     }
 
-    /**
-     * @param fileAge The fileAge in milliseconds to set.
-     */
     public void setFileAge(long fileAge)
     {
         this.fileAge = fileAge;
         this.checkFileAge = true;
     }
 
-    /**
-     * @return Returns the writeToDirectory.
-     */
     public String getWriteToDirectory()
     {
         return writeToDirectoryName;
     }
 
-    /**
-     * @param dir The writeToDirectory to set.
-     */
     public void setWriteToDirectory(String dir) throws IOException
     {
         this.writeToDirectoryName = dir;
         if (writeToDirectoryName != null)
         {
             File writeToDirectory = FileUtils.openDirectory(writeToDirectoryName);
-            if (!(writeToDirectory.canRead()) || !writeToDirectory.canWrite())
+            if (!writeToDirectory.canWrite())
             {
                 throw new IOException(
-                        "Error on initialization, Write To directory does not exist or is not read/write");
+                        "Error on initialization, " + writeToDirectory
+                        + " does not exist or is not writeable");
             }
         }
     }
 
-    /**
-     * @return Returns the readFromDirectory.
-     */
     public String getReadFromDirectory()
     {
         return readFromDirectoryName;
     }
 
-    /**
-     * @param dir The readFromDirectory to set.
-     */
     public void setReadFromDirectory(String dir) throws IOException
     {
         this.readFromDirectoryName = dir;
         if (readFromDirectoryName != null)
         {
-            File readFromDirectory = FileUtils.openDirectory((readFromDirectoryName));
-            if (!readFromDirectory.canRead())
-            {
-                throw new IOException(
-                        "Error on initialization, read from directory does not exist or is not readable");
-            }
+            // check if the directory exists/can be read
+            FileUtils.openDirectory((readFromDirectoryName));
         }
     }
 
@@ -514,16 +507,17 @@ public class FileConnector extends AbstractConnector
      * will be called only when Streaming is being used on an outbound endpoint
      *
      * @param endpoint the endpoint that releates to this Dispatcher
-     * @param message  the current message being processed
+     * @param event  the current event being processed
      * @return the output stream to use for this request or null if the transport
      *         does not support streaming
      * @throws org.mule.api.MuleException
      */
-    public OutputStream getOutputStream(ImmutableEndpoint endpoint, MuleMessage message)
-            throws MuleException
+    @Override
+    public OutputStream getOutputStream(OutboundEndpoint endpoint, MuleEvent event) throws MuleException
     {
+        MuleMessage message = event.getMessage();
         String address = endpoint.getEndpointURI().getAddress();
-        String writeToDirectory = message.getStringProperty(FileConnector.PROPERTY_WRITE_TO_DIRECTORY, null);
+        String writeToDirectory = message.getOutboundProperty(FileConnector.PROPERTY_WRITE_TO_DIRECTORY);
         if (writeToDirectory == null)
         {
             writeToDirectory = getWriteToDirectory();
@@ -537,7 +531,7 @@ public class FileConnector extends AbstractConnector
         String outPattern = (String) endpoint.getProperty(FileConnector.PROPERTY_OUTPUT_PATTERN);
         if (outPattern == null)
         {
-            outPattern = message.getStringProperty(FileConnector.PROPERTY_OUTPUT_PATTERN, null);
+            outPattern = message.getOutboundProperty(FileConnector.PROPERTY_OUTPUT_PATTERN);
         }
         if (outPattern == null)
         {
@@ -551,7 +545,7 @@ public class FileConnector extends AbstractConnector
             }
             else
             {
-                filename = message.getStringProperty(FileConnector.PROPERTY_FILENAME, null);
+                filename = message.getOutboundProperty(FileConnector.PROPERTY_FILENAME);
                 if (filename == null)
                 {
                     filename = generateFilename(message, null);
@@ -572,8 +566,25 @@ public class FileConnector extends AbstractConnector
         }
         catch (IOException e)
         {
-            throw new DispatchException(CoreMessages.streamingFailedNoStream(), message,
-                    endpoint, e);
+            throw new DispatchException(CoreMessages.streamingFailedNoStream(), event, endpoint, e);
+        }
+    }
+
+    protected void move(final File sourceFile, File destinationFile) throws DefaultMuleException
+    {
+        if (destinationFile != null)
+        {
+            // move sourceFile to new destination. Do not use FileUtils here as it ultimately
+            // falls back to copying the file which will cause problems on large files again -
+            // which is what we're trying to avoid in the first place
+            boolean fileWasMoved = sourceFile.renameTo(destinationFile);
+
+            // move didn't work - bail out
+            if (!fileWasMoved)
+            {
+                throw new DefaultMuleException(FileMessages.failedToMoveFile(sourceFile.getAbsolutePath(),
+                    destinationFile.getAbsolutePath()));
+            }
         }
     }
 
@@ -596,19 +607,26 @@ public class FileConnector extends AbstractConnector
         this.streaming = streaming;
     }
 
-    public MessageAdapter getMessageAdapter(Object message) throws MessagingException
+    @Override
+    public MuleMessageFactory createMuleMessageFactory() throws CreateException
     {
+        // See MULE-3209, MULE-3199
         if (isStreaming())
         {
-            // TODO Shouldn't we have a way to specify MessageAdaptor for streaming
-            // in service descriptor
-            // See MULE-3209, MULE-3199
-            return new FileMessageAdapter(message);
+            return new FileMuleMessageFactory(muleContext);
         }
         else
         {
-            return super.getMessageAdapter(message);
+            return super.createMuleMessageFactory();
         }
     }
+    public boolean isRecursive()
+    {
+        return recursive;
+    }
 
+    public void setRecursive(boolean recursive)
+    {
+        this.recursive = recursive;
+    }
 }
