@@ -1,5 +1,5 @@
 /*
- * $Id: SoapAttachmentsFunctionalTestCase.java 22156 2011-06-08 21:36:30Z dfeist $
+ * $Id: SoapAttachmentsFunctionalTestCase.java 23497 2011-12-18 11:46:14Z dfeist $
  * --------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
  *
@@ -10,29 +10,39 @@
 
 package org.mule.transport.soap.axis.functional;
 
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertNotNull;
+
 import org.mule.DefaultMuleEvent;
 import org.mule.DefaultMuleMessage;
 import org.mule.api.MuleMessage;
 import org.mule.api.endpoint.OutboundEndpoint;
 import org.mule.message.ds.StringDataSource;
-import org.mule.session.DefaultMuleSession;
-import org.mule.tck.DynamicPortTestCase;
-import org.mule.transport.AbstractConnector;
+import org.mule.tck.junit4.FunctionalTestCase;
+import org.mule.tck.junit4.rule.DynamicPort;
 import org.mule.transport.soap.axis.AxisMessageDispatcher;
 
 import javax.activation.DataHandler;
 
-public class SoapAttachmentsFunctionalTestCase extends DynamicPortTestCase
+import org.junit.Rule;
+import org.junit.Test;
+
+public class SoapAttachmentsFunctionalTestCase extends FunctionalTestCase
 {
     private static final int SEND_COUNT = 5;
     
     private int callbackCount = 0;
 
+    @Rule
+    public DynamicPort dynamicPort = new DynamicPort("port1");
+
+    @Override
     protected String getConfigResources()
     {
         return "axis-soap-attachments.xml";
     }
 
+    @Test
     public void testSend() throws Exception
     {
         sendTestData(SEND_COUNT);
@@ -49,8 +59,7 @@ public class SoapAttachmentsFunctionalTestCase extends DynamicPortTestCase
         {
             MuleMessage msg = new DefaultMuleMessage("testPayload", muleContext);
             msg.addOutboundAttachment("testAttachment", new DataHandler(new StringDataSource("foo")));
-            DefaultMuleSession session = new DefaultMuleSession(msg, ((AbstractConnector) ep.getConnector()).getSessionHandler(), muleContext);
-            DefaultMuleEvent event = new DefaultMuleEvent(msg, getTestInboundEndpoint("test://test"), session);
+            DefaultMuleEvent event = new DefaultMuleEvent(msg, getTestInboundEndpoint("test://test"), null);
             MuleMessage result = client.process(event).getMessage();
             assertNotNull(result);
             assertNotNull(result.getPayload());
@@ -59,9 +68,4 @@ public class SoapAttachmentsFunctionalTestCase extends DynamicPortTestCase
         }
     }
 
-    @Override
-    protected int getNumPortsToFind()
-    {
-        return 1;
-    }
 }
