@@ -1,7 +1,7 @@
 /*
- * $Id: VMMessageDispatcherFactory.java 11079 2008-02-27 15:52:01Z tcarlson $
+ * $Id: VMMessageDispatcherFactory.java 19191 2010-08-25 21:05:23Z tcarlson $
  * --------------------------------------------------------------------------------------
- * Copyright (c) MuleSource, Inc.  All rights reserved.  http://www.mulesource.com
+ * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
  *
  * The software in this package is published under the terms of the CPAL v1.0
  * license, a copy of which has been included with this distribution in the
@@ -20,11 +20,7 @@ import org.mule.transport.AbstractMessageDispatcherFactory;
  */
 public class VMMessageDispatcherFactory extends AbstractMessageDispatcherFactory
 {
-    /*
-     * (non-Javadoc)
-     * 
-     * @see org.mule.api.transport.MessageDispatcherFactory#create(org.mule.api.transport.Connector)
-     */
+    @Override
     public MessageDispatcher create(OutboundEndpoint endpoint) throws MuleException
     {
         return new VMMessageDispatcher(endpoint);
