@@ -1,5 +1,5 @@
 /*
- * $Id: NoPersistenceTestCase.java 21762 2011-05-03 01:29:28Z mike.schilling $
+ * $Id: NoPersistenceTestCase.java 22409 2011-07-14 05:14:27Z dirk.olmes $
  * --------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
  *
@@ -20,7 +20,7 @@ public class NoPersistenceTestCase extends AbstractTransactionQueueManagerTestCa
     protected TransactionalQueueManager createQueueManager() throws Exception
     {
         TransactionalQueueManager mgr = new TransactionalQueueManager();
-        mgr.setDefaultQueueConfiguration(new QueueConfiguration(0, new SimpleMemoryObjectStore()));
+        mgr.setDefaultQueueConfiguration(new QueueConfiguration(0, new SimpleMemoryObjectStore<Serializable>()));
         return mgr;
     }
 
